@@ -19,8 +19,6 @@
 ##  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ##  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 ##  SOFTWARE.
-#
-# Version ##VERSION##; see changelog for revision history
 
 try:
     import gtk
@@ -48,7 +46,7 @@ import threading
 import textwrap
 
 from reportbug.exceptions import NoPackage, NoBugs, NoNetwork, NoReport
-from reportbug import debianbts
+from reportbug import debbugs
 from reportbug.urlutils import launch_browser
 
 ISATTY = True
@@ -210,50 +208,26 @@ class ReportViewerDialog (gtk.Dialog):
 # BTS
 
 class Bug (object):
-    def __init__ (self, raw):
-        # Skip the '#'
-        raw = raw[1:]
-        bits = re.split(r'[: ]', raw, 2)
-        self.id, self.tag, self.data = bits
-        # Remove [ and ]
-        self.tag = self.tag[1:-1]
-        self.data = self.data.strip ()
-        self.package = self.data.split(']', 1)[0][1:]
-
-        self.reporter = self.get_data ("Reported by:")
-        self.date = self.get_data ("Date:")
-        self.severity = self.get_data("Severity:").capitalize ()
-        self.version = self.get_data ("Found in version")
-        self.filed_date = self.get_data ("Filed")
-        self.modified_date = self.get_data ("Modified")
-
-        # Get rid of [package] which has been stored in self.package
-        self.info = self.data.split(']', 1)[1]
-        self.info = self.info[:self.info.lower().index ("reported by:")].strip ()
-        if not self.info:
-            self.info = '(no subject)'   
-
-    def get_data (self, token):
-        info = ''
-        try:
-            index = self.data.lower().index (token.lower ())
-        except:
-            return '(unknown)'
-
-        i = index + len(token)
-        while True:
-            c = self.data[i]
-            if c == ';':
-                break
-            info += c
-            i += 1
-        return info.strip ()
+    """Encapsulate a bug report for the GTK+ UI"""
+    def __init__ (self, bug):
+        self.id = bug.bug_num
+        self.tag = u', '.join(bug.tags)
+        self.package = bug.package
+        self.status = bug.pending
+        self.reporter = bug.originator
+        self.date = bug.date
+        self.severity = bug.severity
+        self.version = u', '.join(bug.found_versions)
+        self.filed_date = bug.date
+        self.modified_date = bug.log_modified
+        self.info = bug.subject
 
     def __iter__ (self):
         yield self.id
         yield self.tag
         yield self.package
         yield self.info
+        yield self.status
         yield self.reporter
         yield self.date
         yield self.severity
@@ -355,6 +329,8 @@ class BugPage (gtk.EventBox, threading.Thread):
         self.timeout = timeout
         self.archived = archived
 
+        self.bug_status = None
+
         vbox = gtk.VBox (spacing=12)
         vbox.pack_start (gtk.Label ("Retrieving bug information."), expand=False)
 
@@ -368,12 +344,13 @@ class BugPage (gtk.EventBox, threading.Thread):
         # Start the progress bar
         gobject.timeout_add (10, self.pulse)
 
-        info = debianbts.get_report (int (self.number), self.timeout,
+        info = debbugs.get_report (int (self.number), self.timeout,
                                      self.bts, mirrors=self.mirrors,
                                      http_proxy=self.http_proxy, archived=self.archived)
         if not info:
             self.application.run_once_in_main_thread (self.not_found)
         else:
+            self.bug_status = info[0]
             self.application.run_once_in_main_thread (self.found, info)
 
     def drop_progressbar (self):
@@ -393,7 +370,7 @@ class BugPage (gtk.EventBox, threading.Thread):
         
     def found (self, info):
         self.drop_progressbar ()
-        desc = info[0].split(':', 1)[1].strip ()
+        desc = info[0].subject
         bodies = info[1]
         vbox = gtk.VBox (spacing=12)
         vbox.set_border_width (12)
@@ -430,11 +407,11 @@ class BugPage (gtk.EventBox, threading.Thread):
         self.show_all ()
             
     def on_open_browser (self, button):
-        launch_browser (debianbts.get_report_url (self.bts, int (self.number), self.archived))
+        launch_browser (debbugs.get_report_url (self.bts, int (self.number), self.archived))
 
     def on_reply (self, button):
         # Return the bug number to reportbug
-        self.application.set_next_value (self.number)
+        self.application.set_next_value (self.bug_status)
         # Forward the assistant to the progress bar
         self.assistant.forward_page ()
         # Though we're only a page, we are authorized to destroy our parent :)
@@ -638,7 +615,8 @@ class GetStringPage (Page):
         return self.entry.get_text ()
 
     def execute (self, prompt, options=None, force_prompt=False, default=''):
-        self.label.set_text (prompt)
+        # Hackish: remove the text needed for textual UIs...
+        self.label.set_text (prompt.replace(' (enter Ctrl+c to exit reportbug without reporting a bug)', ''))
         self.entry.set_text (default)
 
         if options:
@@ -872,7 +850,7 @@ class HandleBTSQueryPage (TreePage):
 
     def sync_pre_operation (self, package, bts, timeout, mirrors=None, http_proxy="", queryonly=False, screen=None,
                             archived='no', source=False, title=None,
-                            version=None, buglist=None, mbox_reader_cmd=None):
+                            version=None, buglist=None, mbox_reader_cmd=None, latest_first=False):
         self.bts = bts
         self.mirrors = mirrors
         self.http_proxy = http_proxy
@@ -883,7 +861,7 @@ class HandleBTSQueryPage (TreePage):
         if queryonly:
             self.page_type = gtk.ASSISTANT_PAGE_CONFIRM
 
-        sysinfo = debianbts.SYSTEMS[bts]
+        sysinfo = debbugs.SYSTEMS[bts]
         root = sysinfo.get('btsroot')
         if not root:
             # do we need to make a dialog for this?
@@ -894,15 +872,15 @@ class HandleBTSQueryPage (TreePage):
             if source:
                 pkgname += ' (source)'
 
-            progress_label = 'Querying %s bug tracking system for reports on %s' % (debianbts.SYSTEMS[bts]['name'], pkgname)
+            progress_label = 'Querying %s bug tracking system for reports on %s' % (debbugs.SYSTEMS[bts]['name'], pkgname)
         else:
-            progress_label = 'Querying %s bug tracking system for reports %s' % (debianbts.SYSTEMS[bts]['name'], ' '.join([str(x) for x in package]))
+            progress_label = 'Querying %s bug tracking system for reports %s' % (debbugs.SYSTEMS[bts]['name'], ' '.join([str(x) for x in package]))
 
 
         self.application.run_once_in_main_thread (self.assistant.set_progress_label, progress_label)
 
         try:
-            (count, sectitle, hierarchy) = debianbts.get_reports (
+            (count, sectitle, hierarchy) = debbugs.get_reports (
                 package, timeout, bts, mirrors=mirrors, version=version,
                 http_proxy=http_proxy, archived=archived, source=source)
 
@@ -921,8 +899,11 @@ class HandleBTSQueryPage (TreePage):
                 for category, bugs in hierarchy:
                     buglist = []
                     for bug in bugs:
-                        buglist.append (Bug (bug))
-                    report.append ((category, buglist))
+                        buglist.append (bug)
+                    # XXX: this needs to be fixed in debianbts; Bugreport are
+                    # not sortable (on bug_num) - see #639458
+                    sorted(buglist, reverse=latest_first)
+                    report.append ((category, map(Bug, buglist)))
 
                 return (report, sectitle), {}
 
@@ -957,7 +938,7 @@ class HandleBTSQueryPage (TreePage):
         self.view = gtk.TreeView ()
         self.view.set_rules_hint (True)
         scrolled = create_scrollable (self.view)
-        self.columns = ['ID', 'Tag', 'Package', 'Description', 'Reporter', 'Date', 'Severity', 'Version',
+        self.columns = ['ID', 'Tag', 'Package', 'Description', 'Status', 'Submitter', 'Date', 'Severity', 'Version',
                         'Filed date', 'Modified date']
         for col in zip (self.columns, range (len (self.columns))):
             column = gtk.TreeViewColumn (col[0], gtk.CellRendererText (), text=col[1])
@@ -1065,7 +1046,7 @@ class ShowReportPage (Page):
             self.page_type = gtk.ASSISTANT_PAGE_CONFIRM
         return args, kwargs
 
-    def execute (self, number, system, mirrors, http_proxy, timeout, queryonly=False, title='', archived='no'):
+    def execute (self, number, system, mirrors, http_proxy, timeout, queryonly=False, title='', archived='no', mbox_reader_cmd=None):
         self.page.number = number
         self.page.bts = system
         self.page.mirrors = mirrors
@@ -1341,6 +1322,12 @@ class ReportbugAssistant (gtk.Assistant):
             if widget.get_label() == 'gtk-go-back':
                 widget.connect ('show', self.on_back_show)
                 return
+            if widget.get_label() == 'gtk-cancel':
+                image = gtk.image_new_from_stock(gtk.STOCK_QUIT,
+                                                 gtk.ICON_SIZE_BUTTON)
+                widget.set_label("_Quit")
+                widget.set_image(image)
+                return
             if widget.get_label() == 'gtk-go-forward':
                 image = gtk.image_new_from_stock (gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_BUTTON)
                 widget.set_label ("_Continue")
@@ -1431,7 +1418,7 @@ class YesNoDialog (ReportbugConnector, gtk.MessageDialog):
         self.destroy ()
 
     def execute_operation (self, msg, yeshelp=None, nohelp=None, default=True, nowrap=False):
-        self.set_markup (msg+"?")
+        self.set_markup (msg)
         if default:
             self.set_default_response (gtk.RESPONSE_YES)
         else:
@@ -1558,18 +1545,3 @@ Falling back to 'text' interface."""
 
 def can_input ():
     return True
-
-def test ():
-    # Write some tests here
-    print get_password ("test")
-    print select_options ('test', 'A', {'a': 'A test'})
-    print get_multiline ('ENTER', empty_ok=True)
-    print get_string ("test")
-    print system ("yes")
-    page = HandleBTSQueryPage (assistant)
-    application.run_once_in_main_thread (page.execute_operation, [('test', (Bug ('#123 [test] [we] we we Reported by: test;' ), Bug ('#123 [test] [we] we we Reported by: test;')))], 'test')
-    return application.get_last_value ()
-
-if __name__ == '__main__':
-    initialize ()
-    test ()
