@@ -1,7 +1,7 @@
 # urwid user interface for reportbug
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   (C) 2006-08 Chris Lawrence
-#   Copyright (C) 2008-2011 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2012 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -22,8 +22,6 @@
 # Portions of this file are licensed under the Lesser GNU Public License
 # (LGPL) Version 2.1 or later.  On Debian systems, this license is available
 # in /usr/share/common-licenses/LGPL
-#
-# $Id: reportbug_ui_urwid.py,v 1.3.2.19 2007-04-17 20:02:41 lawrencc Exp $
 
 import sys
 import re
@@ -156,7 +154,7 @@ class dialog(object):
                 while not keys:
                     keys = self.ui.get_input()
                 for k in keys:
-                    if urwid.is_mouse_event(k):
+                    if urwid.util.is_mouse_event(k):
                         event, button, col, row = k
                         self.view.mouse_event( size,
                                                event, button, col, row,
@@ -493,18 +491,18 @@ def select_multiple(par, options, prompt, title=None, order=None, extras=None):
 # Things that are very UI dependent go here
 def show_report(number, system, mirrors,
                 http_proxy, timeout, screen=None, queryonly=False, title='',
-                archived='no'):
-    from reportbug import debianbts
+                archived='no', mbox_reader_cmd=None):
+    from reportbug import debbugs
 
     ui = screen
     if not ui:
         ui = initialize_urwid_ui()
 
-    sysinfo = debianbts.SYSTEMS[system]
+    sysinfo = debbugs.SYSTEMS[system]
     display_message('Retrieving report #%d from %s bug tracking system...',
         number, sysinfo['name'], title=title, ui=ui)
 
-    info = debianbts.get_report(number, timeout, system, mirrors=mirrors,
+    info = debbugs.get_report(number, timeout, system, mirrors=mirrors,
                                 http_proxy=http_proxy, archived=archived)
     if not info:
         long_message('Bug report #%d not found.', number, title=title, ui=ui)
@@ -515,28 +513,28 @@ def show_report(number, system, mirrors,
     valid = 'Odmq'
 
     while 1:
-        (bugtitle, bodies) = info
+        (buginfo, bodies) = info
         body = bodies[0]
 
-        r = select_options(body, valid, title=bugtitle, ui=ui, help=options)
+        r = select_options(body, valid, title=buginfo.subject, ui=ui, help=options)
         ui = None
         if not r or (r == 'o'):
             break
         elif r == 'q':
             return -1
         elif r == 'm':
-            return number
+            return buginfo
 
-        launch_browser(debianbts.get_report_url(system, number, archived))
+        launch_browser(debbugs.get_report_url(system, number, archived))
     return
 
 def handle_bts_query(package, bts, timeout, mirrors=None, http_proxy="",
                      queryonly=False, screen=None, title="", archived='no',
                      source=False, version=None, mbox=False, buglist=None,
-                     mbox_reader_cmd=None):
-    from reportbug import debianbts
+                     mbox_reader_cmd=None, latest_first=False):
+    from reportbug import debbugs
 
-    sysinfo = debianbts.SYSTEMS[bts]
+    sysinfo = debbugs.SYSTEMS[bts]
     root = sysinfo.get('btsroot')
     if not root:
         ewrite("%s bug tracking system has no web URL; bypassing query.\n",
@@ -553,16 +551,16 @@ def handle_bts_query(package, bts, timeout, mirrors=None, http_proxy="",
             pkgname += ' (source)'
 
         display_message('Querying %s bug tracking system for reports on %s',
-                        debianbts.SYSTEMS[bts]['name'], pkgname,
+                        debbugs.SYSTEMS[bts]['name'], pkgname,
                         ui=ui, title=title)
     else:
         display_message('Querying %s bug tracking system for reports %s',
-                        debianbts.SYSTEMS[bts]['name'],
+                        debbugs.SYSTEMS[bts]['name'],
                         ' '.join([str(x) for x in package]), ui=ui,title=title)
 
     result = None
     try:
-        (count, sectitle, hierarchy) = debianbts.get_reports(
+        (count, sectitle, hierarchy) = debbugs.get_reports(
             package, timeout, bts, mirrors=mirrors, version=version,
             http_proxy=http_proxy, archived=archived, source=source)
 
@@ -582,20 +580,15 @@ def handle_bts_query(package, bts, timeout, mirrors=None, http_proxy="",
             for (t, bugs) in hierarchy:
                 bcount = len(bugs)
                 buglist.append( ('---', t) )
+                buglist_tmp = {}
                 for bug in bugs:
-                    # encode the bug summary line, to avoid crashes due to
-                    # unparsable UTF8 characters
-                    bug = bug.encode('us-ascii', 'replace')
-                    bits = re.split(r'[: ]', bug[1:], 1)
-                    if len(bits) > 1:
-                        tag, info = bits
-                        info = info.strip()
-                        if not info:
-                            info = '(no subject)'
-                    else:
-                        tag = bug[1:]
-                        info = '(no subject)'
-                    buglist.append( (tag, info) )
+                    # show if the bugs is already resolved
+                    done = ''
+                    if bug.pending == 'done':
+                        done = '  [RESOLVED]'
+                    buglist_tmp[bug.bug_num] = bug.subject+done
+                # append the sorted list of bugs for this severity
+                map(buglist.append, [(str(k), buglist_tmp[k]) for k in sorted(buglist_tmp, reverse=latest_first)])
 
             p = buglist[1][0]
             #scr.popWindow()
@@ -616,8 +609,12 @@ def handle_bts_query(package, bts, timeout, mirrors=None, http_proxy="",
                 if info < 0:
                     if info == -1:
                         result = None
+                    # -2 is the Quit response, triggers the exiting way in main
+                    elif info == -2:
+                        raise NoReport
                     else:
-                        result = info
+                        # uniform to return Bugreport instance
+                        result = debbugs.get_report(info, timeout)[0]
                     break
                 else:
                     p = info
@@ -636,7 +633,9 @@ def handle_bts_query(package, bts, timeout, mirrors=None, http_proxy="",
         long_message('No record of this package found.', title=title)
         raise NoPackage
 
-    if result and result < 0:
+    # we didn't find a report; we access Bugreport thru debbugs,
+    # so to avoid and import of debianbts
+    if result and not isinstance(result, debbugs.debianbts.Bugreport):
         raise NoReport
 
     return result
@@ -681,9 +680,9 @@ def test():
                   'Select mailer: ', default='mutt', empty_ok=True)
     print >> fp, mailer
 
-    import debianbts
+    import debbugs
 
-    tags = debianbts.get_tags()
+    tags = debbugs.get_tags()
 
     taglist = select_multiple(
         'Do any of the following apply to this report?', tags,
